@@ -29,9 +29,10 @@ from hamster import dispatcher, storage, SHARED_DATA_DIR, stuff
 from hamster.charting import Chart
 from hamster.add_custom_fact import CustomFactController
 
-import datetime  as dt
+import datetime as dt
 import calendar
 import gobject
+import time
 
 class StatsViewer:
     def __init__(self):
@@ -51,9 +52,13 @@ class StatsViewer:
         nameColumn.set_cell_data_func(nameCell, self.parent_painter)
         self.fact_tree.append_column(nameColumn)
 
-        self.fact_tree.append_column(gtk.TreeViewColumn("", gtk.CellRendererText(), text=2))
+        timeColumn = gtk.TreeViewColumn(_("Duration"))
+        timeCell = gtk.CellRendererText()
+        timeColumn.pack_end(timeCell, True)
+        timeColumn.set_cell_data_func(timeCell, self.duration_painter)
+        self.fact_tree.append_column(timeColumn)
         
-        self.fact_store = gtk.TreeStore(int, str, str, str) #id, caption, duration, date (invisible)
+        self.fact_store = gtk.TreeStore(int, str, str, str, str) #id, caption, duration, date (invisible), description
         self.fact_tree.set_model(self.fact_store)
         
         x_offset = 80 # let's nicely align all graphs
@@ -89,8 +94,12 @@ class StatsViewer:
         place.add(eventBox)
         
         self.view_date = dt.date.today()
-        self.start_date = self.view_date - dt.timedelta(self.view_date.weekday()) #set to monday
-        self.end_date = self.start_date + dt.timedelta(6) #set to monday
+        
+        self.start_date = self.view_date - dt.timedelta(self.view_date.weekday() + 1) #set to monday
+        # look if we need to start on sunday or monday
+        self.start_date = self.start_date + dt.timedelta(self.locale_first_weekday())
+        
+        self.end_date = self.start_date + dt.timedelta(6)
 
         
         self.day_view = self.get_widget("day")
@@ -114,6 +123,25 @@ class StatsViewer:
         self.fact_tree.grab_focus()
         self.do_graph()
 
+    def locale_first_weekday(self):
+        """figure if week starts on monday or sunday"""
+        import os
+        first_weekday = 6 #by default settle on monday
+
+        try:
+            process = os.popen("locale first_weekday week-1stday")
+            week_offset, week_start = process.read().split('\n')[:2]
+            process.close()
+            week_start = dt.date(*time.strptime(week_start, "%Y%m%d")[:3])
+            week_offset = dt.timedelta(int(week_offset) - 1)
+            beginning = week_start + week_offset
+            first_weekday = int(beginning.strftime("%w"))
+        except:
+            print "WARNING - Failed to get first weekday from locale"
+            pass
+            
+        return first_weekday
+        
     def parent_painter(self, column, cell, model, iter):
         cell_text = model.get_value(iter, 1)
         if model.iter_parent(iter) == None:
@@ -125,49 +153,69 @@ class StatsViewer:
             cell.set_property('markup', text)
 
         else:
-            cell.set_property('text', "   " + cell_text)
-            
-        return
+            activity_name = stuff.escape_pango(cell_text)
+            description = stuff.escape_pango(model.get_value(iter, 4))
+    
+            text = "   %s" % activity_name
+            if description:
+                text+= """\n             <span style="italic" size="small">%s</span>""" % (description)
+                
+            cell.set_property('markup', text)
+
+    def duration_painter(self, column, cell, model, iter):
+        text = model.get_value(iter, 2)
+        if model.iter_parent(iter) == None:
+            if model.get_path(iter) == (0,):
+                text = '<span weight="heavy">%s</span>' % text
+            else:
+                text = '<span weight="heavy" rise="-20000">%s</span>' % text
+        cell.set_property('markup', text)
 
     def get_facts(self):
         self.fact_store.clear()
         totals = {}
-        
+
         by_activity = {}
         by_category = {}
         by_day = {}
-        
+
         week = {"days": [], "totals": []}
-        
+
         facts = storage.get_facts(self.start_date, self.end_date)
-        
+
         for i in range((self.end_date - self.start_date).days  + 1):
             current_date = self.start_date + dt.timedelta(i)
+            # date format in overview window fact listing
+            # prefix is "o_",letter after prefix is regular python format. you can use all of them
+            fact_date = _("%(o_A)s, %(o_b)s %(o_d)s") %  stuff.dateDict(current_date, "o_")
+
             day_row = self.fact_store.append(None, [-1,
-                                                    current_date.strftime('%A, %b %d.'),
+                                                    fact_date,
                                                     "",
-                                                    current_date.strftime('%Y-%m-%d')])
+                                                    current_date.strftime('%Y-%m-%d'),
+                                                    ""])
             by_day[self.start_date + dt.timedelta(i)] = {"duration": 0, "row_pointer": day_row}
-        
+
         for fact in facts:
             start_date = fact["start_time"].date()
-            
+
             duration = None
             if fact["end_time"]: # not set if just started
                 delta = fact["end_time"] - fact["start_time"]
                 duration = 24 * delta.days + delta.seconds / 60
             elif fact["start_time"].date() == dt.date.today():
                 delta = dt.datetime.now() - fact["start_time"]
-                duration = 24 * delta.days + delta.seconds / 60                
+                duration = 24 * delta.days + delta.seconds / 60
 
             self.fact_store.append(by_day[start_date]["row_pointer"],
                                    [fact["id"],
                                     fact["start_time"].strftime('%H:%M') + " " +
                                     fact["name"],
                                     stuff.format_duration(duration),
-                                    fact["start_time"].strftime('%Y-%m-%d')
+                                    fact["start_time"].strftime('%Y-%m-%d'),
+                                    fact["description"]
                                     ])
-            
+
             if fact["name"] not in by_activity: by_activity[fact["name"]] = 0
             if fact["category"] not in by_category: by_category[fact["category"]] = 0
 
@@ -175,20 +223,30 @@ class StatsViewer:
                 by_day[start_date]["duration"] += duration
                 by_activity[fact["name"]] += duration
                 by_category[fact["category"]] += duration
-            
-        days = 7 if self.week_view.get_active() else 30
+
+        days = 30
+        if self.week_view.get_active():
+            days = 7
 
 
         date_sort = lambda a, b: (b[4] < a[4]) - (a[4] < b[4])
         totals["by_day"] = []
 
         for day in by_day:
+            self.fact_store.set_value(by_day[day]["row_pointer"], 2,
+                stuff.format_duration(by_day[day]["duration"]))
             if (self.end_date - self.start_date).days < 20:
-                strday = day.strftime('%a')
+                strday = stuff.locale_to_utf8(day.strftime('%a'))
                 totals["by_day"].append([strday, by_day[day]["duration"] / 60.0, None, None, day])
             else:
-                strday = day.strftime('%d. %b')
-                background = 7 if day.weekday() in [5, 6] else None
+                # date format in month chart in overview window (click on "month" to see it)
+                # prefix is "m_", letter after prefix is regular python format. you can use all of them
+                strday = _("%(m_b)s %(m_d)s") %  stuff.dateDict(day, "m_")
+
+                background = None
+                if day.weekday() in [5, 6]:
+                    background = 7
+
                 totals["by_day"].append([strday, by_day[day]["duration"] / 60.0, None, background, day])
         totals["by_day"].sort(date_sort)
             
@@ -227,28 +285,40 @@ class StatsViewer:
         
 
     def do_graph(self):
+        dates_dict = stuff.dateDict(self.start_date, "start_")
+        dates_dict.update(stuff.dateDict(self.end_date, "end_"))
+        
+        
         if self.start_date.year != self.end_date.year:
-            start_str = self.start_date.strftime('%B %d. %Y')
-            end_str = self.end_date.strftime('%B %d. %Y')
+            # overview label if start and end years don't match
+            # letter after prefixes (start_, end_) is the one of
+            # standard python date formatting ones- you can use all of them
+            overview_label = _(u"Overview for %(start_B)s %(start_d)s, %(start_Y)s – %(end_B)s %(end_d)s, %(end_Y)s") % dates_dict
         elif self.start_date.month != self.end_date.month:
-            start_str = self.start_date.strftime('%B %d.')
-            end_str = self.end_date.strftime('%B %d.')
+            #overview label if start and end month do not match
+            # letter after prefixes (start_, end_) is the one of
+            # standard python date formatting ones- you can use all of them
+            overview_label = _(u"Overview for %(start_B)s %(start_d)s – %(end_B)s %(end_d)s, %(end_Y)s") % dates_dict
         else:
-            start_str = self.start_date.strftime('%B %d')
-            end_str = self.end_date.strftime('%d, %Y')
+            #overview label for interval in same month
+            # letter after prefixes (start_, end_) is the one of
+            # standard python date formatting ones- you can use all of them
+            overview_label = _(u"Overview for %(start_B)s %(start_d)s – %(end_d)s, %(end_Y)s") % dates_dict
 
-        if self.day_view.get_active(): #single day is an exception
-            label_text = _("Overview for %s") % (self.start_date.strftime('%B %d. %Y'))
+        if self.day_view.get_active():
+            # overview label for single day
+            # letter after prefixes (start_, end_) is the one of
+            # standard python date formatting ones- you can use all of them
+            overview_label = _("Overview for %(start_B)s %(start_d)s, %(start_Y)s") % dates_dict
             dayview_caption = _("Day")
         elif self.week_view.get_active():
-            label_text = _("Overview for %s - %s") % (start_str, end_str)
             dayview_caption = _("Week")
         else:
-            label_text = _("Overview for %s - %s") % (start_str, end_str)
             dayview_caption = _("Month")
         
+        
         label = self.get_widget("overview_label")
-        label.set_text(label_text)
+        label.set_text(overview_label)
 
         label2 = self.get_widget("dayview_caption")
         label2.set_markup("<b>%s</b>" % (dayview_caption))
@@ -308,13 +378,14 @@ class StatsViewer:
             self.end_date = self.view_date
         
         elif self.week_view.get_active():
-            self.start_date = self.view_date - dt.timedelta(self.view_date.weekday()) #set to monday
+            self.start_date = self.view_date - dt.timedelta(self.view_date.weekday() + 1)
+            self.start_date = self.start_date + dt.timedelta(self.locale_first_weekday())
             self.end_date = self.start_date + dt.timedelta(6)
         
         elif self.month_view.get_active():
             self.start_date = self.view_date - dt.timedelta(self.view_date.day - 1) #set to beginning of month
             first_weekday, days_in_month = calendar.monthrange(self.view_date.year, self.view_date.month)
-            self.end_date = self.start_date + dt.timedelta(days_in_month - 1) #set to monday
+            self.end_date = self.start_date + dt.timedelta(days_in_month - 1)
         
         self.do_graph()
         
@@ -324,7 +395,9 @@ class StatsViewer:
         self.do_graph()
 
     def on_week_toggled(self, button):
-        self.start_date = self.view_date - dt.timedelta(self.view_date.weekday()) #set to monday
+        self.start_date = self.view_date - dt.timedelta(self.view_date.weekday() + 1)
+        self.start_date = self.start_date + dt.timedelta(self.locale_first_weekday())
+
         self.end_date = self.start_date + dt.timedelta(6)
         self.do_graph()
 
@@ -332,7 +405,7 @@ class StatsViewer:
     def on_month_toggled(self, button):
         self.start_date = self.view_date - dt.timedelta(self.view_date.day - 1) #set to beginning of month
         first_weekday, days_in_month = calendar.monthrange(self.view_date.year, self.view_date.month)
-        self.end_date = self.start_date + dt.timedelta(days_in_month - 1) #set to monday
+        self.end_date = self.start_date + dt.timedelta(days_in_month - 1)
 
         self.do_graph()
         
@@ -381,6 +454,7 @@ class StatsViewer:
             id = model[iter][0]
 
         self.get_widget('remove').set_sensitive(id != -1)
+        self.get_widget('edit').set_sensitive(id != -1)
 
         return True
 
@@ -421,7 +495,9 @@ class StatsViewer:
         return False
 
     def on_window_key_pressed(self, tree, event_key):
-      if (event_key.keyval == gtk.keysyms.Escape):
+      if (event_key.keyval == gtk.keysyms.Escape
+          or (event_key.keyval == gtk.keysyms.w 
+              and event_key.state & gtk.gdk.CONTROL_MASK)):
         self.window.destroy()
     
     def show(self):
